@@ -31,6 +31,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <totem-disc.h>
 #include <stdlib.h>
 #include <math.h>
 
@@ -65,7 +66,6 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
-#include "totem-disc.h"
 
 #include "debug.h"
 
@@ -92,7 +92,7 @@
 
 static const GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0 },
-	{ "_NETSCAPE_URL", 0, 1 },
+	{ "_NETSCAPE_URL", 0, 1 }
 };
 
 static gboolean totem_action_open_files (Totem *totem, char **list);
@@ -157,11 +157,10 @@ totem_action_save_size (Totem *totem)
 }
 
 static void
-totem_action_save_state (Totem *totem)
+totem_action_save_state (Totem *totem, const char *page_id)
 {
 	GKeyFile *keyfile;
 	char *contents, *filename;
-	const char *page_id;
 
 	if (totem->win == NULL)
 		return;
@@ -181,7 +180,6 @@ totem_action_save_state (Totem *totem)
 	g_key_file_set_integer (keyfile, "State",
 			"sidebar_w", totem->sidebar_w);
 
-	page_id = totem_sidebar_get_current_page (totem);
 	g_key_file_set_string (keyfile, "State",
 			"sidebar_page", page_id);
 
@@ -198,6 +196,7 @@ void
 totem_action_exit (Totem *totem)
 {
 	GdkDisplay *display = NULL;
+	char *page_id;
 
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
@@ -213,6 +212,9 @@ totem_action_exit (Totem *totem)
 	if (totem->prefs != NULL)
 		gtk_widget_hide (totem->prefs);
 
+	/* Save the page ID before we close the plugins, otherwise
+	 * we'll never save it properly */
+	page_id = totem_sidebar_get_current_page (totem);
 	totem_object_plugins_shutdown ();
 
 	if (display != NULL)
@@ -222,7 +224,7 @@ totem_action_exit (Totem *totem)
 		int vol;
 
 		vol = bacon_video_widget_get_volume (totem->bvw) * 100.0 + 0.5;
-		//FIXME move the volume to the static file?
+		/* FIXME move the volume to the static file? */
 		gconf_client_set_int (totem->gc,
 				GCONF_PREFIX"/volume",
 				CLAMP (vol, 0, 100),
@@ -230,8 +232,10 @@ totem_action_exit (Totem *totem)
 		totem_action_save_size (totem);
 	}
 
-	bacon_message_connection_free (totem->conn);
-	totem_action_save_state (totem);
+	if (totem->conn != NULL)
+		bacon_message_connection_free (totem->conn);
+	totem_action_save_state (totem, page_id);
+	g_free (page_id);
 
 	totem_sublang_exit (totem);
 	totem_destroy_file_filters ();
@@ -351,7 +355,7 @@ void
 totem_action_show_properties (Totem *totem)
 {
 	if (totem_is_fullscreen (totem) == FALSE)
-		totem_sidebar_set_current_page (totem, "properties");
+		totem_sidebar_set_current_page (totem, "properties", TRUE);
 }
 
 void
@@ -419,9 +423,9 @@ totem_action_seek (Totem *totem, double pos)
 }
 
 void
-totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
+totem_action_set_mrl_and_play (Totem *totem, const char *mrl, const char *subtitle)
 {
-	if (totem_action_set_mrl (totem, mrl) != FALSE)
+	if (totem_action_set_mrl (totem, mrl, subtitle) != FALSE)
 		totem_action_play (totem);
 }
 
@@ -449,11 +453,12 @@ totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 	g_slist_free (filenames);
 
 	if (play != FALSE) {
-		char *mrl;
+		char *mrl, *subtitle;
 
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
 
 	return TRUE;
@@ -551,8 +556,8 @@ totem_action_play_media_device (Totem *totem, const char *device)
 	char *mrl;
 
 	if (totem_action_load_media_device (totem, device) != FALSE) {
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, NULL);
+		totem_action_set_mrl_and_play (totem, mrl, NULL);
 		g_free (mrl);
 	}
 }
@@ -563,8 +568,8 @@ totem_action_play_media (Totem *totem, TotemDiscMediaType type, const char *devi
 	char *mrl;
 
 	if (totem_action_load_media (totem, type, device) != FALSE) {
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, NULL);
+		totem_action_set_mrl_and_play (totem, mrl, NULL);
 		g_free (mrl);
 	}
 }
@@ -581,16 +586,17 @@ totem_action_play_pause (Totem *totem)
 {
 	if (totem->mrl == NULL)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		/* Try to pull an mrl from the playlist */
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 		if (mrl == NULL) {
 			play_pause_set_label (totem, STATE_STOPPED);
 			return;
 		} else {
-			totem_action_set_mrl_and_play (totem, mrl);
+			totem_action_set_mrl_and_play (totem, mrl, subtitle);
 			g_free (mrl);
+			g_free (subtitle);
 			return;
 		}
 	}
@@ -718,16 +724,17 @@ totem_open_location_response_cb (GtkDialog *dialog, gint response, Totem *totem)
 
 	if (uri != NULL)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 		const char *filenames[2];
 
 		filenames[0] = uri;
 		filenames[1] = NULL;
 		totem_action_open_files (totem, (char **) filenames);
 
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
  	g_free (uri);
 
@@ -867,8 +874,10 @@ update_mrl_label (Totem *totem, const char *name)
 }
 
 gboolean
-totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
-		gboolean warn)
+totem_action_set_mrl_with_warning (Totem *totem,
+				   const char *mrl, 
+				   const char *subtitle,
+				   gboolean warn)
 {
 	gboolean retval = TRUE;
 
@@ -914,16 +923,18 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 	} else {
 		gboolean caps;
 		gdouble volume;
-		char *subtitle_uri = NULL;
+		char *autoload_sub = NULL;
 		GError *err = NULL;
 
 		bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 
-		if (totem->autoload_subs != FALSE)
-			subtitle_uri = totem_uri_get_subtitle_uri (mrl);
+		if (subtitle == NULL && totem->autoload_subs != FALSE)
+			autoload_sub = totem_uri_get_subtitle_uri (mrl);
+
 		totem_gdk_window_set_waiting_cursor (totem->win->window);
-		retval = bacon_video_widget_open_with_subtitle (totem->bvw,
-				mrl, subtitle_uri, &err);
+		retval = bacon_video_widget_open_with_subtitle (totem->bvw, mrl,
+								subtitle ? subtitle : autoload_sub, &err);
+		g_free (autoload_sub);
 		gdk_window_set_cursor (totem->win->window, NULL);
 		totem->mrl = g_strdup (mrl);
 
@@ -982,9 +993,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 }
 
 gboolean
-totem_action_set_mrl (Totem *totem, const char *mrl)
+totem_action_set_mrl (Totem *totem, const char *mrl, const char *subtitle)
 {
-	return totem_action_set_mrl_with_warning (totem, mrl, TRUE);
+	return totem_action_set_mrl_with_warning (totem, mrl, subtitle, TRUE);
 }
 
 static gboolean
@@ -1018,11 +1029,13 @@ totem_action_direction (Totem *totem, TotemPlaylistDirection dir)
 			|| bacon_video_widget_is_seekable (totem->bvw) == FALSE
 			|| totem_time_within_seconds (totem) != FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		totem_playlist_set_direction (totem->playlist, dir);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
+
+		g_free (subtitle);
 		g_free (mrl);
 	} else {
 		totem_action_seek (totem, 0);
@@ -1285,14 +1298,15 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 	/* ... and reconnect because we're nice people */
 	if (cleared != FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		g_signal_connect (G_OBJECT (totem->playlist),
 				"changed", G_CALLBACK (playlist_changed_cb),
 				totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
 		g_free (mrl);
+		g_free (subtitle);
 	}
 
 	return TRUE;
@@ -1368,7 +1382,7 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
 	gchar *old_mrl, *new_mrl;
 
-	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist));
+	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist), NULL);
 	new_mrl = totem_pl_parser_resolve_url (old_mrl, mrl);
 	g_free (old_mrl);
 
@@ -1856,12 +1870,13 @@ totem_action_next_angle (Totem *totem)
 void
 totem_action_set_playlist_index (Totem *totem, guint index)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	totem_playlist_set_current (totem->playlist, index);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
-	totem_action_set_mrl_and_play (totem, mrl);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
 	g_free (mrl);
+	g_free (subtitle);
 }
 
 void
@@ -1914,14 +1929,14 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		if (url == NULL) {
 			bacon_video_widget_close (totem->bvw);
 			totem_file_closed (totem);
-			totem_action_set_mrl (totem, NULL);
+			totem_action_set_mrl (totem, NULL, NULL);
 			break;
 		}
 		if (strcmp (url, "dvd:") == 0) {
-			//FIXME b0rked
+			/* FIXME b0rked */
 			totem_action_play_media (totem, MEDIA_TYPE_DVD, NULL);
 		} else if (strcmp (url, "vcd:") == 0) {
-			//FIXME b0rked
+			/* FIXME b0rked */
 			totem_action_play_media (totem, MEDIA_TYPE_VCD, NULL);
 		} else if (g_str_has_prefix (url, "dvb:") != FALSE) {
 			totem_action_play_media (totem, MEDIA_TYPE_DVB, NULL);
@@ -2005,7 +2020,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		totem_action_eject (totem);
 		break;
 	case TOTEM_REMOTE_COMMAND_PLAY_DVD:
-		// TODO - how to see if can, and play the DVD (like the menu item)
+		/* TODO - how to see if can, and play the DVD (like the menu item) */
 		break;
 	case TOTEM_REMOTE_COMMAND_MUTE:
 		totem_action_volume_relative (totem, -1.0);
@@ -2067,17 +2082,19 @@ gboolean totem_action_remote_get_setting (Totem *totem,
 static void
 playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	update_buttons (totem);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 
 	if (mrl == NULL)
 		return;
 
 	if (totem_playlist_get_playing (totem->playlist) == TOTEM_PLAYLIST_STATUS_NONE)
-		totem_action_set_mrl_and_play (totem, mrl);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
+
 	g_free (mrl);
+	g_free (subtitle);
 }
 
 static void
@@ -2089,23 +2106,38 @@ item_activated_cb (GtkWidget *playlist, Totem *totem)
 static void
 current_removed_cb (GtkWidget *playlist, Totem *totem)
 {
-	char *mrl;
+	char *mrl, *subtitle;
 
 	/* Set play button status */
 	play_pause_set_label (totem, STATE_STOPPED);
-	mrl = totem_playlist_get_current_mrl (totem->playlist);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 
-	if (mrl == NULL)
-	{
+	if (mrl == NULL) {
+		g_free (subtitle);
+		subtitle = NULL;
 		totem_playlist_set_at_start (totem->playlist);
 		update_buttons (totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
 	} else {
 		update_buttons (totem);
 	}
 
-	totem_action_set_mrl_and_play (totem, mrl);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
 	g_free (mrl);
+	g_free (subtitle);
+}
+
+static void
+subtitle_changed_cb (GtkWidget *playlist, Totem *totem)
+{
+	char *mrl, *subtitle;
+
+	totem_action_stop (totem);
+	mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+	totem_action_set_mrl_and_play (totem, mrl, subtitle);
+
+	g_free (mrl);
+	g_free (subtitle);
 }
 
 static void
@@ -2211,16 +2243,17 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 	if (totem_playlist_has_next_mrl (totem->playlist) == FALSE
 			&& totem_playlist_get_repeat (totem->playlist) == FALSE)
 	{
-		char *mrl;
+		char *mrl, *subtitle;
 
 		/* Set play button status */
 		totem_playlist_set_at_start (totem->playlist);
 		update_buttons (totem);
 		totem_action_stop (totem);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_with_warning (totem, mrl, FALSE);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_with_warning (totem, mrl, subtitle, FALSE);
 		bacon_video_widget_pause (totem->bvw);
 		g_free (mrl);
+		g_free (subtitle);
 	} else {
 		totem_action_next (totem);
 	}
@@ -2648,7 +2681,7 @@ main_pane_size_allocated (GtkWidget *main_pane, GtkAllocation *allocation, Totem
 	}
 }
 
-static void
+static char *
 totem_setup_window (Totem *totem)
 {
 	GKeyFile *keyfile;
@@ -2734,7 +2767,7 @@ totem_setup_window (Totem *totem)
 		gtk_widget_modify_bg (vbox, i, &black);
 
 	totem_sidebar_setup (totem, show_sidebar, page_id);
-        g_free (page_id);
+	return page_id;
 }
 
 static void
@@ -2865,23 +2898,27 @@ playlist_widget_setup (Totem *totem)
 	gtk_widget_show_all (GTK_WIDGET (totem->playlist));
 
 	g_signal_connect (G_OBJECT (totem->playlist), "active-name-changed",
-			G_CALLBACK (on_playlist_change_name), totem);
+			  G_CALLBACK (on_playlist_change_name), totem);
 	g_signal_connect (G_OBJECT (totem->playlist), "item-activated",
-			G_CALLBACK (item_activated_cb), totem);
+			  G_CALLBACK (item_activated_cb), totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"changed", G_CALLBACK (playlist_changed_cb),
-			totem);
+			  "changed", G_CALLBACK (playlist_changed_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"current-removed", G_CALLBACK (current_removed_cb),
-			totem);
+			  "current-removed", G_CALLBACK (current_removed_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"repeat-toggled",
-			G_CALLBACK (playlist_repeat_toggle_cb),
-			totem);
+			  "repeat-toggled",
+			  G_CALLBACK (playlist_repeat_toggle_cb),
+			  totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
-			"shuffle-toggled",
-			G_CALLBACK (playlist_shuffle_toggle_cb),
-			totem);
+			  "shuffle-toggled",
+			  G_CALLBACK (playlist_shuffle_toggle_cb),
+			  totem);
+	g_signal_connect (G_OBJECT (totem->playlist),
+			  "subtitle-changed",
+			  G_CALLBACK (subtitle_changed_cb),
+			  totem);
 }
 
 static void
@@ -2891,7 +2928,7 @@ video_widget_create (Totem *totem)
 	GtkContainer *container;
 	BaconVideoWidget **bvw;
 	const GtkTargetEntry source_table[] = {
-		{ "text/uri-list", 0, 0 },
+		{ "text/uri-list", 0, 0 }
 	};
 
 	totem->bvw = BACON_VIDEO_WIDGET
@@ -3025,6 +3062,7 @@ main (int argc, char **argv)
 #endif
 	GOptionContext *context;
 	GOptionGroup *baconoptiongroup;
+	char *sidebar_pageid;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -3079,14 +3117,18 @@ main (int argc, char **argv)
 #endif /* !HAVE_GTK_ONLY */
 
 	totem = g_object_new (TOTEM_TYPE_OBJECT, NULL);
+	totem->gc = gc;
 
 	/* IPC stuff */
-	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
-	totem->gc = gc;
-	if (bacon_message_connection_get_is_server (totem->conn) == FALSE) {
-		totem_options_process_for_server (totem->conn, &optionstate);
-		gdk_notify_startup_complete ();
-		totem_action_exit (totem);
+	if (optionstate.notconnectexistingsession == FALSE) {
+		totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
+		if (bacon_message_connection_get_is_server (totem->conn) == FALSE) {
+			totem_options_process_for_server (totem->conn, &optionstate);
+			gdk_notify_startup_complete ();
+			totem_action_exit (totem);
+		} else {
+			totem_options_process_early (totem, &optionstate);
+		}
 	} else {
 		totem_options_process_early (totem, &optionstate);
 	}
@@ -3122,7 +3164,7 @@ main (int argc, char **argv)
 	totem_setup_file_filters ();
 	totem_setup_play_disc (totem);
 	totem_callback_connect (totem);
-	totem_setup_window (totem);
+	sidebar_pageid = totem_setup_window (totem);
 
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
@@ -3155,17 +3197,18 @@ main (int argc, char **argv)
 	/* Command-line handling */
 	totem_options_process_late (totem, &optionstate);
 
-	/* Initialise all the plugins */
+	/* Initialise all the plugins, and set the default page, in case
+	 * it comes from a plugin */
 	totem_object_plugins_init (totem);
-	//FIXME we should set the current page again, in case a plugin was
-	// the default page
+	totem_sidebar_set_current_page (totem, sidebar_pageid, FALSE);
+	g_free (sidebar_pageid);
 
 	if (totem->session_restored != FALSE) {
 		totem_session_restore (totem, optionstate.filenames);
 	} else if (optionstate.filenames != NULL && totem_action_open_files (totem, optionstate.filenames)) {
 		totem_action_play_pause (totem);
 	} else {
-		totem_action_set_mrl (totem, NULL);
+		totem_action_set_mrl (totem, NULL, NULL);
 	}
 
 	/* Set the logo at the last minute so we won't try to show it before a video */
@@ -3174,7 +3217,7 @@ main (int argc, char **argv)
 	if (optionstate.fullscreen == FALSE)
 		gdk_window_set_cursor (totem->win->window, NULL);
 
-	if (bacon_message_connection_get_is_server (totem->conn) != FALSE)
+	if (totem->conn != NULL && bacon_message_connection_get_is_server (totem->conn) != FALSE)
 	{
 		bacon_message_connection_set_callback (totem->conn,
 				(BaconMessageReceivedFunc)
